@@ -39,37 +39,37 @@
 /**********************************************************************/
 /* Exported Variables                                                 */
 /**********************************************************************/
-gint         game_state = STATE_NOT_PLAYING;
+gint game_state = STATE_PLAYING;
 /**********************************************************************/
 
 
 /**********************************************************************/
 /* File Static Variables                                              */
 /**********************************************************************/
-static gint      num_robots1    = 0;
-static gint      num_robots2    = 0;
-static gint      endlev_counter = 0;
-static gint      current_level  = 0;
-static gint      score          = 0;
-static gint      kills          = 0;
-static gint      score_step     = 0;
-static gint      safe_teleports = 0;
-static gboolean  display_updated = 0;
-static gint      player_xpos    = 0;
-static gint      player_ypos    = 0;
-static gint      push_xpos      = -1;
-static gint      push_ypos      = -1;
-static gint      game_timer_id  = -1;
-static gint      arena[GAME_WIDTH][GAME_HEIGHT];
-static gint      old_arena[GAME_WIDTH][GAME_HEIGHT];
-static gint      temp_arena[GAME_WIDTH][GAME_HEIGHT];
+static gint num_robots1 = 0;
+static gint num_robots2 = 0;
+static gint endlev_counter = 0;
+static gint current_level = 0;
+static gint score = 0;
+static gint kills = 0;
+static gint score_step = 0;
+static gint safe_teleports = 0;
+static gboolean display_updated = 0;
+static gint player_xpos = 0;
+static gint player_ypos = 0;
+static gint push_xpos = -1;
+static gint push_ypos = -1;
+static gint game_timer_id = -1;
+static gint arena[GAME_WIDTH][GAME_HEIGHT];
+static gint old_arena[GAME_WIDTH][GAME_HEIGHT];
+static gint temp_arena[GAME_WIDTH][GAME_HEIGHT];
 /**********************************************************************/
 
 
 /**********************************************************************/
 /* Function Prototypes                                                */
 /**********************************************************************/
-static void message_box (gchar *msg);
+static void message_box (gchar * msg);
 static guint log_score (gint sc);
 static void add_kill (gint type);
 static void clear_arena (void);
@@ -80,7 +80,6 @@ static void update_arena (void);
 static gint timeout_cb (void *data);
 static void destroy_game_timer (void);
 static void create_game_timer (void);
-static void create_demo_objects (void);
 static void move_all_robots (void);
 static void move_type2_robots (void);
 static void move_robots (void);
@@ -88,6 +87,7 @@ static gboolean check_safe (gint x, gint y);
 static gboolean push_heap (gint x, gint y, gint dx, gint dy);
 static gboolean try_player_move (gint dx, gint dy);
 static gboolean safe_move_available (void);
+static gboolean safe_teleport_available (void);
 static gboolean player_move (gint dx, gint dy);
 static gboolean random_teleport (void);
 static gboolean safe_teleport (void);
@@ -106,15 +106,14 @@ static gboolean safe_teleport (void);
  * Displays a modal dialog box with a given message
  **/
 static void
-message_box (gchar *msg)
+message_box (gchar * msg)
 {
   GtkWidget *box;
 
-  box = gnome_app_message (GNOME_APP (app),
-                           msg);
-
-  gtk_window_set_modal (GTK_WINDOW (box), TRUE);
-  gtk_widget_show (box);
+  box = gtk_message_dialog_new (GTK_WINDOW (app), GTK_DIALOG_MODAL,
+				GTK_MESSAGE_INFO, GTK_BUTTONS_OK, msg);
+  gtk_dialog_run (GTK_DIALOG (box));
+  gtk_widget_destroy (box);
 }
 
 /**
@@ -132,21 +131,27 @@ show_scores (guint pos)
   gchar *nbuf = NULL;
 
   if (properties_super_safe_moves ()) {
-    sbuf = g_strdup_printf ("%s-super-safe", game_config_filename (current_game_config ()));
+    sbuf =
+      g_strdup_printf ("%s-super-safe",
+		       game_config_filename (current_game_config ()));
   } else if (properties_safe_moves ()) {
-    sbuf = g_strdup_printf ("%s-safe", game_config_filename (current_game_config ()));
+    sbuf =
+      g_strdup_printf ("%s-safe",
+		       game_config_filename (current_game_config ()));
   } else {
-    sbuf = g_strdup_printf ("%s", game_config_filename (current_game_config ()));
+    sbuf =
+      g_strdup_printf ("%s", game_config_filename (current_game_config ()));
   }
 
   if (properties_super_safe_moves ()) {
-    nbuf = g_strdup_printf (_("'%s' with super-safe moves"), 
-                            _(game_config_name (current_game_config ())));
+    nbuf = g_strdup_printf (_("'%s' with super-safe moves"),
+			    _(game_config_name (current_game_config ())));
   } else if (properties_safe_moves ()) {
-    nbuf = g_strdup_printf (_("'%s' with safe moves"), 
-                            _(game_config_name (current_game_config ())));
+    nbuf = g_strdup_printf (_("'%s' with safe moves"),
+			    _(game_config_name (current_game_config ())));
   } else {
-    nbuf = g_strdup_printf ("'%s'", _(game_config_name (current_game_config ())));
+    nbuf =
+      g_strdup_printf ("'%s'", _(game_config_name (current_game_config ())));
   }
 
   if (score_dialog != NULL) {
@@ -156,9 +161,17 @@ show_scores (guint pos)
   score_dialog = gnome_scores_display (nbuf, GAME_NAME, sbuf, pos);
   g_free (sbuf);
   g_free (nbuf);
+
+  if (!score_dialog)
+    return;
+
   gtk_window_set_transient_for (GTK_WINDOW (score_dialog), GTK_WINDOW (app));
+  gtk_dialog_set_has_separator (GTK_DIALOG (score_dialog), FALSE);
+  gtk_container_set_border_width (GTK_CONTAINER (score_dialog), 5);
+  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (score_dialog)->vbox), 2);
+  gtk_window_set_resizable (GTK_WINDOW (score_dialog), FALSE);
   g_signal_connect (G_OBJECT (score_dialog), "destroy",
-                    G_CALLBACK (gtk_widget_destroyed), &score_dialog);
+		    G_CALLBACK (gtk_widget_destroyed), &score_dialog);
 }
 
 
@@ -175,25 +188,47 @@ show_scores (guint pos)
 static guint
 log_score (gint sc)
 {
-  guint pos;
+  guint pos = 0;
   gchar *sbuf = NULL;
 
   if (properties_super_safe_moves ()) {
-    sbuf = g_strdup_printf ("%s-super-safe", game_config_filename (current_game_config ()));  
-  } else
-    if (properties_safe_moves ()) {
-      sbuf = g_strdup_printf ("%s-safe", game_config_filename (current_game_config ()));
-    } else {
-      sbuf = g_strdup_printf ("%s", game_config_filename (current_game_config ()));
-    }
+    sbuf =
+      g_strdup_printf ("%s-super-safe",
+		       game_config_filename (current_game_config ()));
+  } else if (properties_safe_moves ()) {
+    sbuf =
+      g_strdup_printf ("%s-safe",
+		       game_config_filename (current_game_config ()));
+  } else {
+    sbuf =
+      g_strdup_printf ("%s", game_config_filename (current_game_config ()));
+  }
 
-  pos = gnome_score_log ((gfloat)sc, sbuf, TRUE);
+  if (sc != 0)
+    pos = gnome_score_log ((gfloat) sc, sbuf, TRUE);
   g_free (sbuf);
-  update_score_state  ();
+  update_score_state ();
 
   return pos;
 }
 
+/**
+ * kill_player
+ *
+ * Description:
+ * Ends the current game.
+ **/
+static void
+kill_player (void)
+{
+  game_state = STATE_DEAD;
+  play_sound (SOUND_DIE);
+  arena[player_xpos][player_ypos] = OBJECT_PLAYER;
+  endlev_counter = 0;
+  add_aieee_bubble (player_xpos, player_ypos);
+  player_animation_dead ();
+  set_move_menu_sensitivity (FALSE);
+}
 
 /**
  * add_kill
@@ -225,26 +260,27 @@ add_kill (gint type)
 
   score += si;
   score_step += si;
-  
+
   if (game_config ()->safe_score_boundary > 0) {
     while (score_step >= game_config ()->safe_score_boundary) {
       safe_teleports += 1;
       score_step -= game_config ()->safe_score_boundary;
     }
   }
-  
+
   if (game_config ()->num_robots_per_safe > 0) {
     while (kills >= game_config ()->num_robots_per_safe) {
       safe_teleports += 1;
       kills -= game_config ()->num_robots_per_safe;
     }
   }
-  
+
   if (safe_teleports > game_config ()->max_safe_teleports) {
     safe_teleports = game_config ()->max_safe_teleports;
   }
-  
-  gnobots_statusbar_set (score, current_level+1, safe_teleports, num_robots1, num_robots2);
+
+  gnobots_statusbar_set (score, current_level + 1, safe_teleports,
+			 num_robots1, num_robots2);
 }
 
 
@@ -270,6 +306,28 @@ clear_arena (void)
   num_robots2 = 0;
 }
 
+/**
+ * load_temp_arena
+ *
+ * Description:
+ * Set up the temporary arena for processing speculative moves. 
+ *
+ **/
+static void
+load_temp_arena (void)
+{
+  gint i, j;
+
+  for (i = 0; i < GAME_WIDTH; ++i) {
+    for (j = 0; j < GAME_HEIGHT; ++j) {
+      if (arena[i][j] != OBJECT_PLAYER) {
+	temp_arena[i][j] = arena[i][j];
+      } else {
+	temp_arena[i][j] = OBJECT_NONE;
+      }
+    }
+  }
+}
 
 /**
  * check_location
@@ -319,7 +377,8 @@ generate_level (void)
   if (num_robots1 > MAX_ROBOTS) {
     current_level = 0;
     num_robots1 = game_config ()->initial_type1;
-    message_box (_("Congratulations, You Have Defeated the Robots!! \nBut Can You do it Again?"));
+    message_box (_
+		 ("Congratulations, You Have Defeated the Robots!! \nBut Can You do it Again?"));
     play_sound (SOUND_VICTORY);
   }
 
@@ -330,11 +389,12 @@ generate_level (void)
     num_robots2 = game_config ()->maximum_type2;
   }
 
-  if ((num_robots1+num_robots2) > MAX_ROBOTS) {
+  if ((num_robots1 + num_robots2) > MAX_ROBOTS) {
     current_level = 0;
     num_robots1 = game_config ()->initial_type1;
     num_robots2 = game_config ()->initial_type2;
-    message_box (_("Congratulations, You Have Defeated the Robots!! \nBut Can You do it Again?"));
+    message_box (_
+		 ("Congratulations, You Have Defeated the Robots!! \nBut Can You do it Again?"));
     play_sound (SOUND_VICTORY);
   }
 
@@ -420,9 +480,9 @@ update_arena (void)
     for (j = 0; j < GAME_HEIGHT; ++j) {
 
 
-      if ( (temp_arena[i][j] == OBJECT_HEAP) && 
-           (push_xpos == i) && (push_ypos == j)) {
-	if (arena[i][j] == OBJECT_ROBOT1) {	
+      if ((temp_arena[i][j] == OBJECT_HEAP) &&
+	  (push_xpos == i) && (push_ypos == j)) {
+	if (arena[i][j] == OBJECT_ROBOT1) {
 	  if (properties_splats ()) {
 	    add_splat_bubble (i, j);
 	    play_sound (SOUND_SPLAT);
@@ -450,17 +510,12 @@ update_arena (void)
     }
   }
 
-  if (game_state != STATE_NOT_PLAYING) {
-    if (arena[player_xpos][player_ypos] != OBJECT_PLAYER) {
-      game_state = STATE_DEAD;
-      play_sound (SOUND_DIE);
-      arena[player_xpos][player_ypos] = OBJECT_PLAYER;
-      endlev_counter = 0;
-      add_aieee_bubble (player_xpos, player_ypos);
-      player_animation_dead ();
-      set_move_menu_sensitivity (FALSE);
-    }
-
+  if (arena[player_xpos][player_ypos] != OBJECT_PLAYER) {
+    kill_player ();
+  } else {
+    /* This is in the else statement to catch the case where the last
+     * two robots collide on top of the human. Without the "else" this
+     * leads to the player being ressurected and winning. */
     if ((num_robots1 + num_robots2) <= 0) {
       game_state = STATE_COMPLETE;
       play_sound (SOUND_YAHOO);
@@ -473,8 +528,9 @@ update_arena (void)
 
   display_updated = FALSE;
 
-  gnobots_statusbar_set (score, current_level+1, safe_teleports, num_robots1, num_robots2);
-  
+  gnobots_statusbar_set (score, current_level + 1, safe_teleports,
+			 num_robots1, num_robots2);
+
 }
 
 
@@ -494,7 +550,7 @@ timeout_cb (void *data)
 
   draw_graphics ();
 
-  if ( (game_state == STATE_TYPE2) || (game_state == STATE_WTYPE2)) {
+  if ((game_state == STATE_TYPE2) || (game_state == STATE_WTYPE2)) {
     if (display_updated) {
       move_type2_robots ();
       update_arena ();
@@ -519,7 +575,8 @@ timeout_cb (void *data)
       generate_level ();
       game_state = STATE_PLAYING;
       set_move_menu_sensitivity (TRUE);
-      gnobots_statusbar_set (score, current_level+1, safe_teleports, num_robots1, num_robots2);
+      gnobots_statusbar_set (score, current_level + 1, safe_teleports,
+			     num_robots1, num_robots2);
     }
   } else if (game_state == STATE_DEAD) {
     ++endlev_counter;
@@ -531,12 +588,7 @@ timeout_cb (void *data)
 	  show_scores (sp);
 	}
       }
-      game_state = STATE_NOT_PLAYING;
-      remove_bubble ();
-      reset_player_animation ();
-      clear_game_area ();
-      create_demo_objects ();      
-      game_state = STATE_NOT_PLAYING;
+      start_new_game ();
     }
   }
 
@@ -577,34 +629,6 @@ create_game_timer (void)
 }
 
 
-/**
- * create_demo_objects
- *
- * Description:
- * Creates objects for the initial 'demo' screen
- **/
-static void
-create_demo_objects (void)
-{
-  clear_arena ();
-
-  arena[22][15] = OBJECT_PLAYER;
-
-  arena[22][9] = OBJECT_ROBOT1;
-  arena[22][21] = OBJECT_ROBOT1;
-
-  arena[28][15] = OBJECT_ROBOT2;
-  arena[16][15] = OBJECT_ROBOT2;
-
-  arena[28][9] = OBJECT_HEAP;
-  arena[28][21] = OBJECT_HEAP;
-  arena[16][9] = OBJECT_HEAP;
-  arena[16][21] = OBJECT_HEAP;
-
-  gnobots_statusbar_set (0, 0, 0, 0, 0);
-}
-
-
 /**
  * init_game
  *
@@ -614,29 +638,26 @@ create_demo_objects (void)
 void
 init_game (void)
 {
-  clear_game_area ();
-
-  create_demo_objects ();
-
   create_game_timer ();
 
   g_signal_connect (GTK_OBJECT (app), "key_press_event",
-                    GTK_SIGNAL_FUNC (keyboard_cb), 0);  
+		    GTK_SIGNAL_FUNC (keyboard_cb), 0);
 
-  set_move_menu_sensitivity (FALSE);
+  start_new_game ();
 }
 
 
 /**
- * cleanup_game
+ * quit_game
  *
  * Description:
- * cleans up all of game resources
+ * Stop animation timeouts and exit.
  **/
 void
-cleanup_game (void)
+quit_game (void)
 {
   destroy_game_timer ();
+  gtk_main_quit ();
 }
 
 
@@ -655,8 +676,11 @@ start_new_game (void)
   kills = 0;
   score_step = 0;
 
+  if (game_state == STATE_PLAYING)
+    log_score (score);
+
   conf = game_config ();
-  g_return_if_fail  (conf != NULL);
+  g_return_if_fail (conf != NULL);
 
   safe_teleports = conf->initial_safe_teleports;
 
@@ -673,7 +697,8 @@ start_new_game (void)
 
   game_state = STATE_PLAYING;
 
-  gnobots_statusbar_set (score, current_level+1, safe_teleports, num_robots1, num_robots2);
+  gnobots_statusbar_set (score, current_level + 1, safe_teleports,
+			 num_robots1, num_robots2);
   set_move_menu_sensitivity (TRUE);
 }
 
@@ -692,9 +717,8 @@ move_all_robots (void)
 
   for (i = 0; i < GAME_WIDTH; ++i) {
     for (j = 0; j < GAME_HEIGHT; ++j) {
-      if ((arena[i][j] == OBJECT_PLAYER) ||
-          (arena[i][j] == OBJECT_HEAP)) {
-	temp_arena[i][j] = arena[i][j];	
+      if ((arena[i][j] == OBJECT_PLAYER) || (arena[i][j] == OBJECT_HEAP)) {
+	temp_arena[i][j] = arena[i][j];
       } else {
 	temp_arena[i][j] = OBJECT_NONE;
       }
@@ -706,15 +730,19 @@ move_all_robots (void)
       if ((arena[i][j] == OBJECT_ROBOT1) || (arena[i][j] == OBJECT_ROBOT2)) {
 	nx = i;
 	ny = j;
-	if (player_xpos < nx) nx -= 1;
-	if (player_xpos > nx) nx += 1;
-	if (player_ypos < ny) ny -= 1;
-	if (player_ypos > ny) ny += 1;
-	
+	if (player_xpos < nx)
+	  nx -= 1;
+	if (player_xpos > nx)
+	  nx += 1;
+	if (player_ypos < ny)
+	  ny -= 1;
+	if (player_ypos > ny)
+	  ny += 1;
+
 	if (temp_arena[nx][ny] == OBJECT_HEAP) {
 	  add_kill (arena[i][j]);
-	} else if ((temp_arena[nx][ny] == OBJECT_ROBOT1) || 
-                   (temp_arena[nx][ny] == OBJECT_ROBOT2)) {
+	} else if ((temp_arena[nx][ny] == OBJECT_ROBOT1) ||
+		   (temp_arena[nx][ny] == OBJECT_ROBOT2)) {
 	  add_kill (arena[i][j]);
 	  add_kill (temp_arena[nx][ny]);
 	  temp_arena[nx][ny] = OBJECT_HEAP;
@@ -743,9 +771,8 @@ move_type2_robots (void)
   for (i = 0; i < GAME_WIDTH; ++i) {
     for (j = 0; j < GAME_HEIGHT; ++j) {
       if ((arena[i][j] == OBJECT_PLAYER) ||
-          (arena[i][j] == OBJECT_ROBOT1) ||
-          (arena[i][j] == OBJECT_HEAP)) {
-	temp_arena[i][j] = arena[i][j];	
+	  (arena[i][j] == OBJECT_ROBOT1) || (arena[i][j] == OBJECT_HEAP)) {
+	temp_arena[i][j] = arena[i][j];
       } else {
 	temp_arena[i][j] = OBJECT_NONE;
       }
@@ -757,15 +784,19 @@ move_type2_robots (void)
       if (arena[i][j] == OBJECT_ROBOT2) {
 	nx = i;
 	ny = j;
-	if (player_xpos < nx) nx -= 1;
-	if (player_xpos > nx) nx += 1;
-	if (player_ypos < ny) ny -= 1;
-	if (player_ypos > ny) ny += 1;
-	
+	if (player_xpos < nx)
+	  nx -= 1;
+	if (player_xpos > nx)
+	  nx += 1;
+	if (player_ypos < ny)
+	  ny -= 1;
+	if (player_ypos > ny)
+	  ny += 1;
+
 	if (temp_arena[nx][ny] == OBJECT_HEAP) {
 	  add_kill (arena[i][j]);
-	} else if ((temp_arena[nx][ny] == OBJECT_ROBOT1) || 
-                   (temp_arena[nx][ny] == OBJECT_ROBOT2)) {
+	} else if ((temp_arena[nx][ny] == OBJECT_ROBOT1) ||
+		   (temp_arena[nx][ny] == OBJECT_ROBOT2)) {
 	  add_kill (arena[i][j]);
 	  add_kill (temp_arena[nx][ny]);
 	  temp_arena[nx][ny] = OBJECT_HEAP;
@@ -821,13 +852,14 @@ check_safe (gint x, gint y)
   gint i, j;
   gint nx, ny;
 
-  if (temp_arena[x][y] != OBJECT_NONE) return FALSE;
+  if (temp_arena[x][y] != OBJECT_NONE)
+    return FALSE;
 
   for (i = 0; i < GAME_WIDTH; ++i) {
     for (j = 0; j < GAME_HEIGHT; ++j) {
       if ((temp_arena[i][j] == OBJECT_PLAYER) ||
-          (temp_arena[i][j] == OBJECT_HEAP)) {
-	temp2_arena[i][j] = temp_arena[i][j];	
+	  (temp_arena[i][j] == OBJECT_HEAP)) {
+	temp2_arena[i][j] = temp_arena[i][j];
       } else {
 	temp2_arena[i][j] = OBJECT_NONE;
       }
@@ -836,18 +868,22 @@ check_safe (gint x, gint y)
 
   for (i = 0; i < GAME_WIDTH; ++i) {
     for (j = 0; j < GAME_HEIGHT; ++j) {
-      if ((temp_arena[i][j] == OBJECT_ROBOT1) || 
-          (temp_arena[i][j] == OBJECT_ROBOT2)) {
+      if ((temp_arena[i][j] == OBJECT_ROBOT1) ||
+	  (temp_arena[i][j] == OBJECT_ROBOT2)) {
 	nx = i;
 	ny = j;
-	if (x < nx) nx -= 1;
-	if (x > nx) nx += 1;
-	if (y < ny) ny -= 1;
-	if (y > ny) ny += 1;
-	
-	if ((temp2_arena[nx][ny] == OBJECT_ROBOT1) || 
-            (temp2_arena[nx][ny] == OBJECT_ROBOT2) ||
-            (temp2_arena[nx][ny] == OBJECT_HEAP)) {
+	if (x < nx)
+	  nx -= 1;
+	if (x > nx)
+	  nx += 1;
+	if (y < ny)
+	  ny -= 1;
+	if (y > ny)
+	  ny += 1;
+
+	if ((temp2_arena[nx][ny] == OBJECT_ROBOT1) ||
+	    (temp2_arena[nx][ny] == OBJECT_ROBOT2) ||
+	    (temp2_arena[nx][ny] == OBJECT_HEAP)) {
 	  temp2_arena[nx][ny] = OBJECT_HEAP;
 	} else {
 	  temp2_arena[nx][ny] = temp_arena[i][j];
@@ -856,13 +892,14 @@ check_safe (gint x, gint y)
     }
   }
 
-  if (temp2_arena[x][y] != OBJECT_NONE) return FALSE;
+  if (temp2_arena[x][y] != OBJECT_NONE)
+    return FALSE;
 
   for (i = 0; i < GAME_WIDTH; ++i) {
     for (j = 0; j < GAME_HEIGHT; ++j) {
       if ((temp2_arena[i][j] == OBJECT_PLAYER) ||
-          (temp2_arena[i][j] == OBJECT_HEAP)) {
-	temp3_arena[i][j] = temp2_arena[i][j];	
+	  (temp2_arena[i][j] == OBJECT_HEAP)) {
+	temp3_arena[i][j] = temp2_arena[i][j];
       } else {
 	temp3_arena[i][j] = OBJECT_NONE;
       }
@@ -874,14 +911,18 @@ check_safe (gint x, gint y)
       if (temp2_arena[i][j] == OBJECT_ROBOT2) {
 	nx = i;
 	ny = j;
-	if (x < nx) nx -= 1;
-	if (x > nx) nx += 1;
-	if (y < ny) ny -= 1;
-	if (y > ny) ny += 1;
-	
-	if ((temp3_arena[nx][ny] == OBJECT_ROBOT1) || 
-            (temp3_arena[nx][ny] == OBJECT_ROBOT2) ||
-            (temp3_arena[nx][ny] == OBJECT_HEAP)) {
+	if (x < nx)
+	  nx -= 1;
+	if (x > nx)
+	  nx += 1;
+	if (y < ny)
+	  ny -= 1;
+	if (y > ny)
+	  ny += 1;
+
+	if ((temp3_arena[nx][ny] == OBJECT_ROBOT1) ||
+	    (temp3_arena[nx][ny] == OBJECT_ROBOT2) ||
+	    (temp3_arena[nx][ny] == OBJECT_HEAP)) {
 	  temp3_arena[nx][ny] = OBJECT_HEAP;
 	} else {
 	  temp3_arena[nx][ny] = temp2_arena[i][j];
@@ -890,7 +931,8 @@ check_safe (gint x, gint y)
     }
   }
 
-  if (temp3_arena[x][y] != OBJECT_NONE) return FALSE;
+  if (temp3_arena[x][y] != OBJECT_NONE)
+    return FALSE;
 
   return TRUE;
 }
@@ -911,16 +953,18 @@ check_safe (gint x, gint y)
 static gboolean
 push_heap (gint x, gint y, gint dx, gint dy)
 {
-  gint nx = x+dx;
-  gint ny = y+dy;
+  gint nx = x + dx;
+  gint ny = y + dy;
+
+  if (temp_arena[x][y] != OBJECT_HEAP)
+    return FALSE;
 
-  if (temp_arena[x][y] != OBJECT_HEAP) return FALSE;
-  
   if ((nx < 0) || (nx >= GAME_WIDTH) || (ny < 0) || (ny >= GAME_HEIGHT)) {
     return FALSE;
   }
 
-  if (temp_arena[nx][ny] == OBJECT_HEAP) return FALSE;
+  if (temp_arena[nx][ny] == OBJECT_HEAP)
+    return FALSE;
 
   push_xpos = nx;
   push_ypos = ny;
@@ -946,7 +990,6 @@ push_heap (gint x, gint y, gint dx, gint dy)
 static gboolean
 try_player_move (gint dx, gint dy)
 {
-  gint i, j;
   gint nx, ny;
 
   nx = player_xpos + dx;
@@ -956,15 +999,7 @@ try_player_move (gint dx, gint dy)
     return FALSE;
   }
 
-  for (i = 0; i < GAME_WIDTH; ++i) {
-    for (j = 0; j < GAME_HEIGHT; ++j) {
-      if (arena[i][j] != OBJECT_PLAYER) {
-	temp_arena[i][j] = arena[i][j];
-      } else {
-	temp_arena[i][j] = OBJECT_NONE;
-      }
-    }
-  }
+  load_temp_arena ();
 
   if (temp_arena[nx][ny] == OBJECT_HEAP) {
     if (game_config ()->moveable_heaps) {
@@ -980,7 +1015,6 @@ try_player_move (gint dx, gint dy)
   return TRUE;
 }
 
-
 /**
  * safe_move_available
  *
@@ -994,23 +1028,23 @@ static gboolean
 safe_move_available (void)
 {
   if (try_player_move (-1, -1)) {
-    if (check_safe (player_xpos-1, player_ypos-1)) {
+    if (check_safe (player_xpos - 1, player_ypos - 1)) {
       return TRUE;
     }
   }
   if (try_player_move (0, -1)) {
-    if (check_safe (player_xpos, player_ypos-1)) {
+    if (check_safe (player_xpos, player_ypos - 1)) {
       return TRUE;
     }
   }
   if (try_player_move (1, -1)) {
-    if (check_safe (player_xpos+1, player_ypos-1)) {
+    if (check_safe (player_xpos + 1, player_ypos - 1)) {
       return TRUE;
     }
   }
 
   if (try_player_move (-1, 0)) {
-    if (check_safe (player_xpos-1, player_ypos)) {
+    if (check_safe (player_xpos - 1, player_ypos)) {
       return TRUE;
     }
   }
@@ -1020,23 +1054,23 @@ safe_move_available (void)
     }
   }
   if (try_player_move (1, 0)) {
-    if (check_safe (player_xpos+1, player_ypos)) {
+    if (check_safe (player_xpos + 1, player_ypos)) {
       return TRUE;
     }
   }
 
   if (try_player_move (-1, 1)) {
-    if (check_safe (player_xpos-1, player_ypos+1)) {
+    if (check_safe (player_xpos - 1, player_ypos + 1)) {
       return TRUE;
     }
   }
   if (try_player_move (0, 1)) {
-    if (check_safe (player_xpos, player_ypos+1)) {
+    if (check_safe (player_xpos, player_ypos + 1)) {
       return TRUE;
     }
   }
   if (try_player_move (1, 1)) {
-    if (check_safe (player_xpos+1, player_ypos+1)) {
+    if (check_safe (player_xpos + 1, player_ypos + 1)) {
       return TRUE;
     }
   }
@@ -1044,6 +1078,32 @@ safe_move_available (void)
   return FALSE;
 }
 
+/**
+ * safe_teleport_available
+ *
+ * Description:
+ * Check for a safe teleport.
+ *
+ * Returns:
+ * TRUE is a safe teleport is possible, FALSE otherwise.
+ *
+ */
+static gboolean
+safe_teleport_available (void)
+{
+  int x, y;
+
+  load_temp_arena ();
+
+  for (x = 0; x < GAME_WIDTH; x++) {
+    for (y = 0; y < GAME_HEIGHT; y++) {
+      if (check_safe (x, y))
+	return TRUE;
+    }
+  }
+
+  return FALSE;
+}
 
 /**
  * player_move
@@ -1076,7 +1136,7 @@ player_move (gint dx, gint dy)
       if (!check_safe (nx, ny)) {
 	if (properties_super_safe_moves () || safe_move_available ()) {
 	  play_sound (SOUND_BAD);
-	  return FALSE;	
+	  return FALSE;
 	}
       }
     }
@@ -1140,7 +1200,7 @@ random_teleport (void)
       temp_arena[player_xpos][player_ypos] = OBJECT_PLAYER;
 
       reset_player_animation ();
-      
+
       update_arena ();
       break;
     }
@@ -1155,7 +1215,8 @@ random_teleport (void)
     }
 
     if ((xp == ixp) && (yp == iyp)) {
-      message_box (_("There are no teleport locations left!!"));      
+      /* This should never happen. */
+      message_box (_("There are no teleport locations left!!"));
       return FALSE;
     }
   }
@@ -1182,10 +1243,25 @@ safe_teleport (void)
   gint xp, yp, ixp, iyp;
   gint i, j;
 
-  if (safe_teleports <= 0) {
-    return random_teleport ();
+  if (properties_super_safe_moves () && !safe_teleport_available ()) {
+    /* FIXME: This code is untested - in normal play you have to get to 
+     * about level 61. */
+    if (!safe_move_available ()) {
+      message_box (_
+		   ("You have run out of safe moves - the robots have won!"));
+      kill_player ();
+      return FALSE;
+    } else if (safe_teleports > 0) {
+      message_box (_("There are no safe locations to teleport to!!"));
+      return FALSE;
+    } else {
+      return random_teleport ();
+    }
   }
-  
+
+  if (safe_teleports <= 0)
+    return random_teleport ();
+
   for (i = 0; i < GAME_WIDTH; ++i) {
     for (j = 0; j < GAME_HEIGHT; ++j) {
       if (arena[i][j] != OBJECT_PLAYER) {
@@ -1222,13 +1298,8 @@ safe_teleport (void)
 	yp = 0;
       }
     }
-
-    if ((xp == ixp) && (yp == iyp)) {
-      message_box (_("There are no safe locations to teleport to!!"));
-      return FALSE;
-    }
   }
-  
+
   remove_splat_bubble ();
   play_sound (SOUND_TELEPORT);
 
@@ -1246,63 +1317,64 @@ safe_teleport (void)
 void
 game_keypress (gint key)
 {
-  if (!display_updated) return;
+  if (!display_updated)
+    return;
 
   if (game_state == STATE_PLAYING) {
     switch (key) {
     case KBD_NW:
       if (player_move (-1, -1)) {
-        move_robots ();
+	move_robots ();
       }
       break;
     case KBD_N:
       if (player_move (0, -1)) {
-        move_robots ();
+	move_robots ();
       }
       break;
     case KBD_NE:
       if (player_move (1, -1)) {
-        move_robots ();
+	move_robots ();
       }
       break;
     case KBD_W:
       if (player_move (-1, 0)) {
-        move_robots ();
+	move_robots ();
       }
       break;
     case KBD_STAY:
       if (player_move (0, 0)) {
-        move_robots ();
+	move_robots ();
       }
       break;
     case KBD_E:
       if (player_move (1, 0)) {
-        move_robots ();
+	move_robots ();
       }
       break;
     case KBD_SW:
       if (player_move (-1, 1)) {
-        move_robots ();
+	move_robots ();
       }
       break;
     case KBD_S:
       if (player_move (0, 1)) {
-        move_robots ();
+	move_robots ();
       }
       break;
     case KBD_SE:
       if (player_move (1, 1)) {
-        move_robots ();
+	move_robots ();
       }
       break;
     case KBD_TELE:
       if (safe_teleport ()) {
-        move_robots ();
+	move_robots ();
       }
       break;
     case KBD_RTEL:
       if (random_teleport ()) {
-        move_robots ();
+	move_robots ();
       }
       break;
     case KBD_WAIT:
@@ -1313,15 +1385,17 @@ game_keypress (gint key)
 
 }
 
-static void get_dir (int ix, int iy, int * odx, int * ody)
+static void
+get_dir (int ix, int iy, int *odx, int *ody)
 {
   int x, y, idx, idy;
   double dx, dy, angle;
   int octant;
   const int movetable[8][2] = { {-1, 0}, {-1, -1}, {0, -1}, {1, -1},
-				{1, 0}, {1, 1}, {0, 1}, {-1, 1} };
-  x = CLAMP(ix/TILE_WIDTH, 0, GAME_WIDTH);
-  y = CLAMP(iy/TILE_HEIGHT, 0, GAME_HEIGHT);
+  {1, 0}, {1, 1}, {0, 1}, {-1, 1}
+  };
+  x = CLAMP (ix / tile_width, 0, GAME_WIDTH);
+  y = CLAMP (iy / tile_height, 0, GAME_HEIGHT);
 
   /* If we click on our man then we assume we hold. */
   if ((x == player_xpos) && (y == player_ypos)) {
@@ -1333,48 +1407,45 @@ static void get_dir (int ix, int iy, int * odx, int * ody)
   /* If the square clicked on is a valid move, go there. */
   idx = x - player_xpos;
   idy = y - player_ypos;
-  if ((ABS(idx) < 2) && (ABS(idy) < 2)) {
+  if ((ABS (idx) < 2) && (ABS (idy) < 2)) {
     *odx = idx;
     *ody = idy;
     return;
   }
 
   /* Otherwise go in the general direction of the mouse click. */
-  dx = ix - (player_xpos + 0.5) * TILE_WIDTH;
-  dy = iy - (player_ypos + 0.5) * TILE_HEIGHT;
+  dx = ix - (player_xpos + 0.5) * tile_width;
+  dy = iy - (player_ypos + 0.5) * tile_height;
 
   angle = atan2 (dy, dx);
 
   /* Note the adjustment we have to make (+9, not +8) because atan2's idea 
    * of octants and the ones we want are shifted by PI/8. */
-  octant = (((int)floor(8.0*angle/M_PI) + 9)/2) % 8; 
+  octant = (((int) floor (8.0 * angle / M_PI) + 9) / 2) % 8;
 
   *odx = movetable[octant][0];
   *ody = movetable[octant][1];
 }
 
-gboolean mouse_cb (GtkWidget * widget, GdkEventButton * e, gpointer data)
+gboolean
+mouse_cb (GtkWidget * widget, GdkEventButton * e, gpointer data)
 {
   int dx, dy;
 
-  if (game_state == STATE_NOT_PLAYING) {
-    start_new_game ();
-    return TRUE;
-  }
-
   if (game_state != STATE_PLAYING)
     return TRUE;
 
   get_dir (e->x, e->y, &dx, &dy);
 
-  player_move (dx, dy);
-
-  move_robots ();
+  if (player_move (dx, dy)) {
+    move_robots ();
+  }
 
   return TRUE;
 }
 
-gboolean move_cb (GtkWidget *widget, GdkEventMotion *e, gpointer data)
+gboolean
+move_cb (GtkWidget * widget, GdkEventMotion * e, gpointer data)
 {
   int dx, dy;
 
